package org.apache.commons.lang3.concurrent;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import org.apache.commons.lang3.concurrent.ConcurrentException;
import org.apache.commons.lang3.concurrent.ConcurrentInitializer;
import org.apache.commons.lang3.concurrent.ConcurrentUtils;

public abstract class BackgroundInitializer<T> implements ConcurrentInitializer<T> {
  private ExecutorService externalExecutor;
  
  private ExecutorService executor;
  
  private Future<T> future;
  
  protected BackgroundInitializer() {
    this(null);
  }
  
  protected BackgroundInitializer(ExecutorService exec) {
    setExternalExecutor(exec);
  }
  
  public final synchronized ExecutorService getExternalExecutor() {
    return this.externalExecutor;
  }
  
  public synchronized boolean isStarted() {
    return (this.future != null);
  }
  
  public final synchronized void setExternalExecutor(ExecutorService externalExecutor) {
    if (isStarted())
      throw new IllegalStateException("Cannot set ExecutorService after start()!"); 
    this.externalExecutor = externalExecutor;
  }
  
  public synchronized boolean start() {
    if (!isStarted()) {
      ExecutorService tempExec;
      this.executor = getExternalExecutor();
      if (this.executor == null) {
        this.executor = tempExec = createExecutor();
      } else {
        tempExec = null;
      } 
      this.future = this.executor.submit(createTask(tempExec));
      return true;
    } 
    return false;
  }
  
  public T get() throws ConcurrentException {
    try {
      return getFuture().get();
    } catch (ExecutionException execex) {
      ConcurrentUtils.handleCause(execex);
      return null;
    } catch (InterruptedException iex) {
      Thread.currentThread().interrupt();
      throw new ConcurrentException(iex);
    } 
  }
  
  public synchronized Future<T> getFuture() {
    if (this.future == null)
      throw new IllegalStateException("start() must be called first!"); 
    return this.future;
  }
  
  protected final synchronized ExecutorService getActiveExecutor() {
    return this.executor;
  }
  
  protected int getTaskCount() {
    return 1;
  }
  
  protected abstract T initialize() throws Exception;
  
  private Callable<T> createTask(ExecutorService execDestroy) {
    return (Callable<T>)new InitializationTask(this, execDestroy);
  }
  
  private ExecutorService createExecutor() {
    return Executors.newFixedThreadPool(getTaskCount());
  }
}
